import React, { Component } from "react";
import merge from "lodash/merge";
import {
    BasicConfig, BasicFuncs, Utils,
    // types:
    Operators, Widgets, Fields, Config, Types, Conjunctions, Settings, LocaleSettings, OperatorProximity, Funcs,
    DateTimeFieldSettings,
} from "react-awesome-query-builder";
import moment from "moment";
import ru_RU from "antd/lib/locale-provider/ru_RU";
import { ruRU } from "@material-ui/core/locale";

import AntdConfig from "react-awesome-query-builder/config/antd";
import AntdWidgets from "react-awesome-query-builder/components/widgets/antd";
import MaterialConfig from "react-awesome-query-builder/config/material";
const {
    FieldSelect,
    FieldDropdown,
    FieldCascader,
    FieldTreeSelect,
} = AntdWidgets;
const { simulateAsyncFetch } = Utils;

const skinToConfig: Record<string, Config> = {
    vanilla: BasicConfig,
    antd: AntdConfig,
    material: MaterialConfig,
};

export default (skin: string) => {
    const InitialConfig = skinToConfig[skin] as BasicConfig;

    const demoListValues = [
        { title: "A", value: "a" },
        { title: "AA", value: "aa" },
        { title: "AAA1", value: "aaa1" },
        { title: "AAA2", value: "aaa2" },
        { title: "B", value: "b" },
        { title: "C", value: "c" },
        { title: "D", value: "d" },
        { title: "E", value: "e" },
        { title: "F", value: "f" },
        { title: "G", value: "g" },
        { title: "H", value: "h" },
        { title: "I", value: "i" },
        { title: "J", value: "j" },
    ];
    const simulatedAsyncFetch = simulateAsyncFetch(demoListValues, 3);

    const conjunctions: Conjunctions = {
        ...InitialConfig.conjunctions,
    };

    const proximity: OperatorProximity = {
        ...InitialConfig.operators.proximity,
        valueLabels: [
            { label: "Word 1", placeholder: "Enter first word" },
            { label: "Word 2", placeholder: "Enter second word" },
        ],
        textSeparators: [
            //'Word 1',
            //'Word 2'
        ],
        options: {
            ...InitialConfig.operators.proximity.options,
            optionLabel: "Near", // label on top of "near" selectbox (for config.settings.showLabels==true)
            optionTextBefore: "Near", // label before "near" selectbox (for config.settings.showLabels==false)
            optionPlaceholder: "Select words between", // placeholder for "near" selectbox
            minProximity: 2,
            maxProximity: 10,
            defaults: {
                proximity: 2
            },
            customProps: {}
        }
    };

    const operators: Operators = {
        ...InitialConfig.operators,
        // examples of  overriding
        proximity,
        between: {
            ...InitialConfig.operators.between,
            valueLabels: [
                "Value from",
                "Value to"
            ],
            textSeparators: [
                "from",
                "to"
            ],
        },
    };


    const widgets: Widgets = {
        ...InitialConfig.widgets,
        // examples of  overriding
        text: {
            ...InitialConfig.widgets.text
        },
        textarea: {
            ...InitialConfig.widgets.textarea,
            maxRows: 3
        },
        slider: {
            ...InitialConfig.widgets.slider
        },
        rangeslider: {
            ...InitialConfig.widgets.rangeslider
        },
        date: {
            ...InitialConfig.widgets.date,
            dateFormat: "DD.MM.YYYY",
            valueFormat: "YYYY-MM-DD",
        },
        time: {
            ...InitialConfig.widgets.time,
            timeFormat: "HH:mm",
            valueFormat: "HH:mm:ss",
        },
        datetime: {
            ...InitialConfig.widgets.datetime,
            timeFormat: "HH:mm",
            dateFormat: "DD.MM.YYYY",
            valueFormat: "YYYY-MM-DD HH:mm:ss",
        },
        func: {
            ...InitialConfig.widgets.func,
            customProps: {
                showSearch: true
            }
        },
        select: {
            ...InitialConfig.widgets.select,
        },
        multiselect: {
            ...InitialConfig.widgets.multiselect,
            customProps: {
                //showCheckboxes: false,
                width: "200px",
                input: {
                    width: "100px"
                }
            }
        },
        treeselect: {
            ...InitialConfig.widgets.treeselect,
            customProps: {
                showSearch: true
            }
        },
    };


    const types: Types = {
        ...InitialConfig.types,
        // examples of  overriding
        boolean: merge(InitialConfig.types.boolean, {
            widgets: {
                boolean: {
                    widgetProps: {
                        hideOperator: true,
                        operatorInlineLabel: "is"
                    },
                    opProps: {
                        equal: {
                            label: "is"
                        },
                        not_equal: {
                            label: "is not"
                        }
                    }
                },
            },
        }),
    };


    const localeSettings: LocaleSettings = {
        locale: {
            moment: "ru",
            antd: ru_RU,
            material: ruRU,
        },
        valueLabel: "Value",
        valuePlaceholder: "Value",
        fieldLabel: "Field",
        operatorLabel: "Operator",
        funcLabel: "Function",
        fieldPlaceholder: "Select field",
        funcPlaceholder: "Select function",
        operatorPlaceholder: "Select operator",
        lockLabel: "Lock",
        lockedLabel: "Locked",
        deleteLabel: null,
        addGroupLabel: "Add group",
        addRuleLabel: "Add rule",
        addSubRuleLabel: "Add sub rule",
        delGroupLabel: null,
        notLabel: "Not",
        valueSourcesPopupTitle: "Select value source",
        removeRuleConfirmOptions: {
            title: "Are you sure delete this rule?",
            okText: "Yes",
            okType: "danger",
            cancelText: "Cancel"
        },
        removeGroupConfirmOptions: {
            title: "Are you sure delete this group?",
            okText: "Yes",
            okType: "danger",
            cancelText: "Cancel"
        },
    };

    const settings: Settings = {
        ...InitialConfig.settings,
        ...localeSettings,

        defaultSliderWidth: "200px",
        defaultSelectWidth: "200px",
        defaultSearchWidth: "100px",
        defaultMaxRows: 5,

        valueSourcesInfo: {
            value: {
                label: "Value"
            },
            field: {
                label: "Field",
                widget: "field",
            },
            func: {
                label: "Function",
                widget: "func",
            }
        },
        // canReorder: true,
        // canRegroup: true,
        // showLock: true,
        // showNot: true,
        // showLabels: true,
        maxNesting: 5,
        canLeaveEmptyGroup: true,
        shouldCreateEmptyGroup: false,
        showErrorMessage: true,
        customFieldSelectProps: {
            showSearch: true
        },
        // renderField: (props) => <FieldCascader {...props} />,
        // renderOperator: (props) => <FieldDropdown {...props} />,
        // renderFunc: (props) => <FieldSelect {...props} />,
        // maxNumberOfRules: 10 // number of rules can be added to the query builder
    };

    //////////////////////////////////////////////////////////////////////

    //   const fields: Fields = {
    //     user: {
    //       label: "User",
    //       tooltip: "Group of fields",
    //       type: "!struct",
    //       subfields: {
    //         firstName: {
    //           label2: "Username", //only for menu's toggler
    //           type: "text",
    //           excludeOperators: ["proximity"],
    //           fieldSettings: {
    //             validateValue: (val: string, fieldSettings) => {
    //               return (val.length < 10);
    //             },
    //           },
    //           mainWidgetProps: {
    //             valueLabel: "Name",
    //             valuePlaceholder: "Enter name",
    //           },
    //         },
    //         login: {
    //           type: "text",
    //           tableName: "t1", // legacy: PR #18, PR #20
    //           excludeOperators: ["proximity"],
    //           fieldSettings: {
    //             validateValue: (val: string, fieldSettings) => {
    //               return (val.length < 10 && (val === "" || val.match(/^[A-Za-z0-9_-]+$/) !== null));
    //             },
    //           },
    //           mainWidgetProps: {
    //             valueLabel: "Login",
    //             valuePlaceholder: "Enter login",
    //           },
    //         }
    //       }
    //     },
    //     bio: {
    //       label: "Bio",
    //       type: "text",
    //       preferWidgets: ["textarea"],
    //       fieldSettings: {
    //         maxLength: 1000,
    //       }
    //     },
    //     results: {
    //       label: "Results",
    //       type: "!group",
    //       subfields: {
    //         product: {
    //           type: "select",
    //           fieldSettings: {
    //             listValues: ["abc", "def", "xyz"],
    //           },
    //           valueSources: ["value"],
    //         },
    //         score: {
    //           type: "number",
    //           fieldSettings: {
    //             min: 0,
    //             max: 100,
    //           },
    //           valueSources: ["value"],
    //         }
    //       }
    //     },
    //     cars: {
    //       label: "Cars",
    //       type: "!group",
    //       mode: "array",
    //       conjunctions: ["AND", "OR"],
    //       showNot: true,
    //       operators: [
    //         // w/ operand - count
    //         "equal",
    //         "not_equal",
    //         "less",
    //         "less_or_equal",
    //         "greater",
    //         "greater_or_equal",
    //         "between",
    //         "not_between",

    //         // w/o operand
    //         "some",
    //         "all",
    //         "none",
    //       ],
    //       defaultOperator: "some",
    //       initialEmptyWhere: true, // if default operator is not in config.settings.groupOperators, true - to set no children, false - to add 1 empty

    //       subfields: {
    //         vendor: {
    //           type: "select",
    //           fieldSettings: {
    //             listValues: ["Ford", "Toyota", "Tesla"],
    //           },
    //           valueSources: ["value"],
    //         },
    //         year: {
    //           type: "number",
    //           fieldSettings: {
    //             min: 1990,
    //             max: 2021,
    //           },
    //           valueSources: ["value"],
    //         }
    //       }
    //     },
    //     prox1: {
    //       label: "prox",
    //       tooltip: "Proximity search",
    //       type: "text",
    //       operators: ["proximity"],
    //     },
    //     num: {
    //       label: "Number",
    //       type: "number",
    //       preferWidgets: ["number"],
    //       fieldSettings: {
    //         min: -1,
    //         max: 5
    //       },
    //       funcs: ["LINEAR_REGRESSION"],
    //     },
    //     slider: {
    //       label: "Slider",
    //       type: "number",
    //       preferWidgets: ["slider", "rangeslider"],
    //       valueSources: ["value", "field"],
    //       fieldSettings: {
    //         min: 0,
    //         max: 100,
    //         step: 1,
    //         marks: {
    //           0: <strong>0%</strong>,
    //           100: <strong>100%</strong>
    //         },
    //         validateValue: (val, fieldSettings) => {
    //           return (val < 50 ? null : "Invalid slider value, see validateValue()");
    //         },
    //       },
    //       //overrides
    //       widgets: {
    //         slider: {
    //           widgetProps: {
    //             valuePlaceholder: "..Slider",
    //           }
    //         },
    //         rangeslider: {
    //           widgetProps: {
    //             valueLabels: [
    //               { label: "Number from", placeholder: "from" },
    //               { label: "Number to", placeholder: "to" },
    //             ],
    //           }
    //         },
    //       },
    //     },
    //     date: {
    //       label: "Date",
    //       type: "date",
    //       valueSources: ["value"],
    //       fieldSettings: {
    //         dateFormat: "DD-MM-YYYY",
    //         validateValue: (val, fieldSettings: DateTimeFieldSettings) => {
    //           // example of date validation
    //           const dateVal = moment(val, fieldSettings.valueFormat);
    //           return dateVal.year() != (new Date().getFullYear()) ? "Please use current year" : null;
    //         },
    //       },
    //     },
    //     time: {
    //       label: "Time",
    //       type: "time",
    //       valueSources: ["value"],
    //       defaultOperator: "between",
    //     },
    //     datetime: {
    //       label: "DateTime",
    //       type: "datetime",
    //       valueSources: ["value", "func"]
    //     },
    //     datetime2: {
    //       label: "DateTime2",
    //       type: "datetime",
    //       valueSources: ["field"]
    //     },
    //     color: {
    //       label: "Color",
    //       type: "select",
    //       valueSources: ["value"],
    //       fieldSettings: {
    //         showSearch: true,
    //         // * old format:
    //         // listValues: {
    //         //     yellow: 'Yellow',
    //         //     green: 'Green',
    //         //     orange: 'Orange'
    //         // },
    //         // * new format:
    //         listValues: [
    //           { value: "yellow", title: "Yellow" },
    //           { value: "green", title: "Green" },
    //           { value: "orange", title: "Orange" }
    //         ],
    //       },
    //     },
    //     color2: {
    //       label: "Color2",
    //       type: "select",
    //       fieldSettings: {
    //         listValues: {
    //           yellow: "Yellow",
    //           green: "Green",
    //           orange: "Orange",
    //           purple: "Purple"
    //         },
    //       }
    //     },
    //     multicolor: {
    //       label: "Colors",
    //       type: "multiselect",
    //       fieldSettings: {
    //         showSearch: true,
    //         listValues: {
    //           yellow: "Yellow",
    //           green: "Green",
    //           orange: "Orange"
    //         },
    //         allowCustomValues: true,
    //       }
    //     },
    //     selecttree: {
    //       label: "Color (tree)",
    //       type: "treeselect",
    //       fieldSettings: {
    //         treeExpandAll: true,
    //         // * deep format (will be auto converted to flat format):
    //         // listValues: [
    //         //     { value: "1", title: "Warm colors", children: [
    //         //         { value: "2", title: "Red" },
    //         //         { value: "3", title: "Orange" }
    //         //     ] },
    //         //     { value: "4", title: "Cool colors", children: [
    //         //         { value: "5", title: "Green" },
    //         //         { value: "6", title: "Blue", children: [
    //         //             { value: "7", title: "Sub blue", children: [
    //         //                 { value: "8", title: "Sub sub blue and a long text" }
    //         //             ] }
    //         //         ] }
    //         //     ] }
    //         // ],
    //         // * flat format:
    //         listValues: [
    //           { value: "1", title: "Warm colors" },
    //           { value: "2", title: "Red", parent: "1" },
    //           { value: "3", title: "Orange", parent: "1" },
    //           { value: "4", title: "Cool colors" },
    //           { value: "5", title: "Green", parent: "4" },
    //           { value: "6", title: "Blue", parent: "4" },
    //           { value: "7", title: "Sub blue", parent: "6" },
    //           { value: "8", title: "Sub sub blue and a long text", parent: "7" },
    //         ],
    //       }
    //     },
    //     multiselecttree: {
    //       label: "Colors (tree)",
    //       type: "treemultiselect",
    //       fieldSettings: {
    //         treeExpandAll: true,
    //         listValues: [
    //           { value: "1", title: "Warm colors", children: [
    //             { value: "2", title: "Red" },
    //             { value: "3", title: "Orange" }
    //           ] },
    //           { value: "4", title: "Cool colors", children: [
    //             { value: "5", title: "Green" },
    //             { value: "6", title: "Blue", children: [
    //               { value: "7", title: "Sub blue", children: [
    //                 { value: "8", title: "Sub sub blue and a long text" }
    //               ] }
    //             ] }
    //           ] }
    //         ]
    //       }
    //     },
    //     autocomplete: {
    //       label: "Autocomplete",
    //       type: "select",
    //       valueSources: ["value"],
    //       fieldSettings: {
    //         asyncFetch: simulatedAsyncFetch,
    //         useAsyncSearch: true,
    //         useLoadMore: true,
    //         forceAsyncSearch: false,
    //         allowCustomValues: false
    //       },
    //     },
    //     stock: {
    //       label: "In stock",
    //       type: "boolean",
    //       defaultValue: true,
    //       mainWidgetProps: {
    //         labelYes: "+",
    //         labelNo: "-"
    //       }
    //     },
    //   };

    const fields: any = {
        "Single Campagins": {
            "label": "Single Campagins",
            "type": "!struct",
            "subfields": {
                "Email Campaign": {
                    "label": "Email Campaign",
                    "type": "multiselect",
                    "operators": [
                        "multiselect_equals",
                        "multiselect_not_equals",
                        "is_null",
                        "is_not_null",
                        "all",
                        "none"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "fieldSettings": {
                        "listValues": [],
                        "allowCustomValues": true
                    }
                },
                "Email Campaign Events": {
                    "label": "Email Campaign Events",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Email Campaign Events": {
                            "label": "Email Campaign Events",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Email Campaign Date": {
                    "label": "Email Campaign Date",
                    "type": "date",
                    "valueSources": [
                        "func",
                        "value"
                    ],
                    "funcs": [
                        "Today",
                        "Last"
                    ]
                },
                "Whatsapp Campaign": {
                    "label": "Whatsapp Campaign",
                    "type": "multiselect",
                    "operators": [
                        "multiselect_equals",
                        "multiselect_not_equals",
                        "is_null",
                        "is_not_null",
                        "all",
                        "none"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "fieldSettings": {
                        "listValues": [],
                        "allowCustomValues": true
                    }
                },
                "Whatsapp Campaign Events": {
                    "label": "Whatsapp Campaign Events",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Whatsapp Campaign Events": {
                            "label": "Whatsapp Campaign Events",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Whatsapp Campaign Date": {
                    "label": "Whatsapp Campaign Date",
                    "type": "date",
                    "valueSources": [
                        "func",
                        "value"
                    ],
                    "funcs": [
                        "Today",
                        "Last"
                    ]
                }
            }
        },
        "Web & App": {
            "label": "Web & App",
            "type": "!struct",
            "subfields": {
                "User Login": {
                    "label": "User Login",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "not_equal",
                        "less",
                        "less_or_equal",
                        "greater",
                        "greater_or_equal",
                        "between",
                        "not_between"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        }
                    }
                },
                "User Logout": {
                    "label": "User Logout",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "not_equal",
                        "less",
                        "less_or_equal",
                        "greater",
                        "greater_or_equal",
                        "between",
                        "not_between"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        }
                    }
                },
                "Total Sessions": {
                    "label": "Total Sessions",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "not_equal",
                        "less",
                        "less_or_equal",
                        "greater",
                        "greater_or_equal",
                        "between",
                        "not_between"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        }
                    }
                },
                "User First Seen": {
                    "label": "User First Seen",
                    "type": "date",
                    "valueSources": [
                        "func",
                        "value"
                    ],
                    "funcs": [
                        "Today",
                        "Last"
                    ]
                },
                "User Last Seen": {
                    "label": "User Last Seen",
                    "type": "date",
                    "valueSources": [
                        "func",
                        "value"
                    ],
                    "funcs": [
                        "Today",
                        "Last"
                    ]
                },
                "Platform": {
                    "label": "Platform",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Platform": {
                            "label": "Platform",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Platform Name": {
                    "label": "Platform Name",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Platform Name": {
                            "label": "Platform Name",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Page Title": {
                    "label": "Page Title",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Page Title": {
                            "label": "Page Title",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Page Url": {
                    "label": "Page Url",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Page Url": {
                            "label": "Page Url",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Screen Name": {
                    "label": "Screen Name",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Screen Name": {
                            "label": "Screen Name",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Screen View": {
                    "label": "Screen View",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Screen View": {
                            "label": "Screen View",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Campaign Name": {
                    "label": "Campaign Name",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Campaign Name": {
                            "label": "Campaign Name",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Campaign Source": {
                    "label": "Campaign Source",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Campaign Source": {
                            "label": "Campaign Source",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Campaign Medium": {
                    "label": "Campaign Medium",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Campaign Medium": {
                            "label": "Campaign Medium",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Referral Source": {
                    "label": "Referral Source",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Referral Source": {
                            "label": "Referral Source",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Referral Medium": {
                    "label": "Referral Medium",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Referral Medium": {
                            "label": "Referral Medium",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Browser Name": {
                    "label": "Browser Name",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Browser Name": {
                            "label": "Browser Name",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Device Brand": {
                    "label": "Device Brand",
                    "type": "multiselect",
                    "operators": [
                        "multiselect_equals",
                        "multiselect_not_equals",
                        "is_null",
                        "is_not_null",
                        "all",
                        "none"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "fieldSettings": {
                        "listValues": [],
                        "allowCustomValues": true
                    }
                }
            }
        },
        "Merchandise": {
            "label": "Merchandise",
            "type": "!struct",
            "subfields": {
                "Financial Status": {
                    "label": "Financial Status",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Financial Status": {
                            "label": "Financial Status",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Fulfillment Status": {
                    "label": "Fulfillment Status",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Fulfillment Status": {
                            "label": "Fulfillment Status",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Purchased at": {
                    "label": "Purchased at",
                    "type": "date",
                    "valueSources": [
                        "func",
                        "value"
                    ],
                    "funcs": [
                        "Today",
                        "Last"
                    ]
                },
                "Lineitem Name": {
                    "label": "Lineitem Name",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Lineitem Name": {
                            "label": "Lineitem Name",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Lineitem Size": {
                    "label": "Lineitem Size",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Lineitem Size": {
                            "label": "Lineitem Size",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Lineitem Discount Name": {
                    "label": "Lineitem Discount Name",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Lineitem Discount Name": {
                            "label": "Lineitem Discount Name",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Lineitem Price": {
                    "label": "Lineitem Price",
                    "type": "number",
                    "valueSources": [
                        "value"
                    ],
                    "preferWidgets": [
                        "number"
                    ]
                },
                "Merchandise Source": {
                    "label": "Merchandise Source",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Merchandise Source": {
                            "label": "Merchandise Source",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Average Product Quantity (Lifetime per fan)": {
                    "label": "Average Product Quantity (Lifetime per fan)",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "not_equal",
                        "less",
                        "less_or_equal",
                        "greater",
                        "greater_or_equal",
                        "between",
                        "not_between"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        }
                    }
                },
                "Total Amount Spent (Lifetime per fan)": {
                    "label": "Total Amount Spent (Lifetime per fan)",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "not_equal",
                        "less",
                        "less_or_equal",
                        "greater",
                        "greater_or_equal",
                        "between",
                        "not_between"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        }
                    }
                },
                "Average Order Value (Lifetime per fan)": {
                    "label": "Average Order Value (Lifetime per fan)",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "not_equal",
                        "less",
                        "less_or_equal",
                        "greater",
                        "greater_or_equal",
                        "between",
                        "not_between"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        }
                    }
                },
                "Merchandise First Seen": {
                    "label": "Merchandise First Seen",
                    "type": "date",
                    "valueSources": [
                        "func",
                        "value"
                    ],
                    "funcs": [
                        "Today",
                        "Last"
                    ]
                },
                "Merchandise Last Seen": {
                    "label": "Merchandise Last Seen",
                    "type": "date",
                    "valueSources": [
                        "func",
                        "value"
                    ],
                    "funcs": [
                        "Today",
                        "Last"
                    ]
                }
            }
        },
        "Ticketing": {
            "label": "Ticketing",
            "type": "!struct",
            "subfields": {
                "Ticket Price": {
                    "label": "Ticket Price",
                    "type": "multiselect",
                    "operators": [
                        "multiselect_equals",
                        "multiselect_not_equals",
                        "is_null",
                        "is_not_null",
                        "all",
                        "none"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "fieldSettings": {
                        "listValues": [],
                        "allowCustomValues": true
                    }
                },
                "Match [Day of Week]": {
                    "label": "Match [Day of Week]",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Match [Day of Week]": {
                            "label": "Match [Day of Week]",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Match Name": {
                    "label": "Match Name",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Match Name": {
                            "label": "Match Name",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Season": {
                    "label": "Season",
                    "type": "multiselect",
                    "operators": [
                        "multiselect_equals",
                        "multiselect_not_equals",
                        "is_null",
                        "is_not_null",
                        "all",
                        "none"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "fieldSettings": {
                        "listValues": [],
                        "allowCustomValues": true
                    }
                },
                "Ticketing Source": {
                    "label": "Ticketing Source",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Ticketing Source": {
                            "label": "Ticketing Source",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Stands": {
                    "label": "Stands",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Stands": {
                            "label": "Stands",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Match Date": {
                    "label": "Match Date",
                    "type": "multiselect",
                    "operators": [
                        "multiselect_equals",
                        "multiselect_not_equals",
                        "is_null",
                        "is_not_null",
                        "all",
                        "none"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "fieldSettings": {
                        "listValues": [],
                        "allowCustomValues": true
                    }
                },
                "Ticketing Order Date": {
                    "label": "Ticketing Order Date",
                    "type": "date",
                    "valueSources": [
                        "func",
                        "value"
                    ],
                    "funcs": [
                        "Today",
                        "Last"
                    ]
                },
                "Discount Coupon": {
                    "label": "Discount Coupon",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Discount Coupon": {
                            "label": "Discount Coupon",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Avg Tickets Per Game (Lifetime)": {
                    "label": "Avg Tickets Per Game (Lifetime)",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "not_equal",
                        "less",
                        "less_or_equal",
                        "greater",
                        "greater_or_equal",
                        "between",
                        "not_between"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        }
                    }
                },
                "Number Of Games Attended (Lifetime)": {
                    "label": "Number Of Games Attended (Lifetime)",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "not_equal",
                        "less",
                        "less_or_equal",
                        "greater",
                        "greater_or_equal",
                        "between",
                        "not_between"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        }
                    }
                },
                "Total Amount Spent (Lifetime)": {
                    "label": "Total Amount Spent (Lifetime)",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "not_equal",
                        "less",
                        "less_or_equal",
                        "greater",
                        "greater_or_equal",
                        "between",
                        "not_between"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        }
                    }
                },
                "Avg Order Value (Lifetime)": {
                    "label": "Avg Order Value (Lifetime)",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "not_equal",
                        "less",
                        "less_or_equal",
                        "greater",
                        "greater_or_equal",
                        "between",
                        "not_between"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        }
                    }
                },
                "Redeemed Status": {
                    "label": "Redeemed Status",
                    "type": "boolean",
                    "defaultValue": true,
                    "valueSources": [
                        "value"
                    ],
                    "mainWidgetProps": {
                        "labelYes": "Yes",
                        "labelNo": "No"
                    }
                },
                "Redeemed Timestamp": {
                    "label": "Redeemed Timestamp",
                    "type": "date",
                    "valueSources": [
                        "func",
                        "value"
                    ],
                    "funcs": [
                        "Today",
                        "Last"
                    ]
                },
                "Transaction Device Source": {
                    "label": "Transaction Device Source",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Transaction Device Source": {
                            "label": "Transaction Device Source",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Device Platform": {
                    "label": "Device Platform",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Device Platform": {
                            "label": "Device Platform",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Payment Gateway": {
                    "label": "Payment Gateway",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Payment Gateway": {
                            "label": "Payment Gateway",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Ticketing First Seen": {
                    "label": "Ticketing First Seen",
                    "type": "date",
                    "valueSources": [
                        "func",
                        "value"
                    ],
                    "funcs": [
                        "Today",
                        "Last"
                    ]
                },
                "Ticketing Last Seen": {
                    "label": "Ticketing Last Seen",
                    "type": "date",
                    "valueSources": [
                        "func",
                        "value"
                    ],
                    "funcs": [
                        "Today",
                        "Last"
                    ]
                }
            }
        },
        "Overview": {
            "label": "Overview",
            "type": "!struct",
            "subfields": {
                "Source": {
                    "label": "Source",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Source": {
                            "label": "Source",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Compare Source": {
                    "label": "Compare Source",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Compare Source": {
                            "label": "Compare Source",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Age Group": {
                    "label": "Age Group",
                    "type": "multiselect",
                    "operators": [
                        "multiselect_equals",
                        "multiselect_not_equals",
                        "is_null",
                        "is_not_null",
                        "all",
                        "none"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "fieldSettings": {
                        "listValues": [],
                        "allowCustomValues": true
                    }
                },
                "Gender": {
                    "label": "Gender",
                    "type": "multiselect",
                    "operators": [
                        "multiselect_equals",
                        "multiselect_not_equals",
                        "is_null",
                        "is_not_null",
                        "all",
                        "none"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "fieldSettings": {
                        "listValues": [],
                        "allowCustomValues": true
                    }
                },
                "City": {
                    "label": "City",
                    "type": "multiselect",
                    "operators": [
                        "multiselect_equals",
                        "multiselect_not_equals",
                        "is_null",
                        "is_not_null",
                        "all",
                        "none"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "fieldSettings": {
                        "listValues": [],
                        "allowCustomValues": true
                    }
                },
                "State": {
                    "label": "State",
                    "type": "multiselect",
                    "operators": [
                        "multiselect_equals",
                        "multiselect_not_equals",
                        "is_null",
                        "is_not_null",
                        "all",
                        "none"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "fieldSettings": {
                        "listValues": [],
                        "allowCustomValues": true
                    }
                },
                "Country": {
                    "label": "Country",
                    "type": "select",
                    "operators": [
                        "select_equals",
                        "select_not_equals",
                        "is_null",
                        "is_not_null",
                        "select_any_in",
                        "select_not_any_in",
                        "all",
                        "none"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "fieldSettings": {
                        "listValues": [],
                        "allowCustomValues": true
                    }
                },
                "Email Present": {
                    "label": "Email Present",
                    "type": "boolean",
                    "defaultValue": true,
                    "valueSources": [
                        "value"
                    ],
                    "mainWidgetProps": {
                        "labelYes": "Yes",
                        "labelNo": "No"
                    }
                },
                "Phone Present": {
                    "label": "Phone Present",
                    "type": "boolean",
                    "defaultValue": true,
                    "valueSources": [
                        "value"
                    ],
                    "mainWidgetProps": {
                        "labelYes": "Yes",
                        "labelNo": "No"
                    }
                },
                "Accepts Marketing": {
                    "label": "Accepts Marketing",
                    "type": "boolean",
                    "defaultValue": true,
                    "valueSources": [
                        "value"
                    ],
                    "mainWidgetProps": {
                        "labelYes": "Yes",
                        "labelNo": "No"
                    }
                },
                "Completion Rate %": {
                    "label": "Completion Rate %",
                    "type": "number",
                    "valueSources": [
                        "value"
                    ],
                    "preferWidgets": [
                        "number"
                    ]
                },
                "Total Interactions": {
                    "label": "Total Interactions",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "not_equal",
                        "less",
                        "less_or_equal",
                        "greater",
                        "greater_or_equal",
                        "between",
                        "not_between"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        }
                    }
                },
                "Total Amount Spent": {
                    "label": "Total Amount Spent",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "not_equal",
                        "less",
                        "less_or_equal",
                        "greater",
                        "greater_or_equal",
                        "between",
                        "not_between"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        }
                    }
                },
                "Fans First Seen": {
                    "label": "Fans First Seen",
                    "type": "date",
                    "valueSources": [
                        "func",
                        "value"
                    ],
                    "funcs": [
                        "Today",
                        "Last"
                    ]
                },
                "Fans Last Seen": {
                    "label": "Fans Last Seen",
                    "type": "date",
                    "valueSources": [
                        "func",
                        "value"
                    ],
                    "funcs": [
                        "Today",
                        "Last"
                    ]
                }
            }
        },
        "Loyalty": {
            "label": "Loyalty",
            "type": "!struct",
            "subfields": {
                "Transaction Type": {
                    "label": "Transaction Type",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Transaction Type": {
                            "label": "Transaction Type",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Program Code": {
                    "label": "Program Code",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "all",
                        "none",
                        "some"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "No. of occurrences": {
                            "label": "No. of occurrences",
                            "type": "number",
                            "valueSources": [
                                "value"
                            ],
                            "preferWidgets": [
                                "number"
                            ]
                        },
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        },
                        "Program Code": {
                            "label": "Program Code",
                            "type": "multiselect",
                            "operators": [
                                "multiselect_equals",
                                "multiselect_not_equals",
                                "is_null",
                                "is_not_null",
                                "all",
                                "none"
                            ],
                            "valueSources": [
                                "value"
                            ],
                            "fieldSettings": {
                                "listValues": [],
                                "allowCustomValues": true
                            },
                            "defaultValue": []
                        }
                    }
                },
                "Points": {
                    "label": "Points",
                    "type": "!group",
                    "mode": "array",
                    "conjunctions": [
                        "AND",
                        "OR"
                    ],
                    "showNot": true,
                    "operators": [
                        "not_equal",
                        "less",
                        "less_or_equal",
                        "greater",
                        "greater_or_equal",
                        "between",
                        "not_between"
                    ],
                    "valueSources": [
                        "value"
                    ],
                    "subfields": {
                        "Time interval": {
                            "label": "Time interval",
                            "type": "date",
                            "valueSources": [
                                "func",
                                "value"
                            ],
                            "funcs": [
                                "Today",
                                "Last"
                            ]
                        }
                    }
                },
                "Transaction Date": {
                    "label": "Transaction Date",
                    "type": "date",
                    "valueSources": [
                        "func",
                        "value"
                    ],
                    "funcs": [
                        "Today",
                        "Last"
                    ]
                }
            }
        }
    }

    //////////////////////////////////////////////////////////////////////

    const funcs: Funcs = {
        ...BasicFuncs
    };


    const config: Config = {
        conjunctions,
        operators,
        widgets,
        types,
        settings,
        fields,
        funcs,
    };

    return config;
};

